@@ -4,19 +4,16 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef CATEGORIESREADER_H
-#define CATEGORIESREADER_H
+#pragma once
 
-#include <QVector>
 #include "discovercommon_export.h"
+#include <QVector>
 
 class Category;
 class AbstractResourcesBackend;
 class DISCOVERCOMMON_EXPORT CategoriesReader
 {
-    public:
-        QVector<Category*> loadCategoriesPath(const QString& path);
-        QVector<Category*> loadCategoriesFile(AbstractResourcesBackend* backend);
+public:
+    QVector<Category *> loadCategoriesPath(const QString &path);
+    QVector<Category *> loadCategoriesFile(AbstractResourcesBackend *backend);
 };
-
-#endif // CATEGORIESREADER_H
