@@ -53,6 +53,7 @@ QHash<int, QByteArray> UpdateModel::roleNames() const
     ret.insert(SectionRole, "section");
     ret.insert(ChangelogRole, "changelog");
     ret.insert(UpgradeTextRole, "upgradeText");
+    ret.insert(ExtendedRole, "extended");
     return ret;
 }
 
@@ -113,7 +114,7 @@ QVariant UpdateModel::data(const QModelIndex &index, int role) const
     case Qt::CheckStateRole:
         return item->checked();
     case SizeRole:
-        return KFormat().formatByteSize(item->size());
+        return item->size() > 0 ? KFormat().formatByteSize(item->size()) : i18n("Unknown");
     case ResourceRole:
         return QVariant::fromValue<QObject *>(item->resource());
     case ResourceProgressRole:
@@ -122,6 +123,8 @@ QVariant UpdateModel::data(const QModelIndex &index, int role) const
         return item->state();
     case ChangelogRole:
         return item->changelog();
+    case ExtendedRole:
+        return item->isExtended();
     case SectionRole: {
         static const QString appUpdatesSection = i18nc("@item:inlistbox", "Applications");
         static const QString systemUpdateSection = i18nc("@item:inlistbox", "System Software");
@@ -182,6 +185,12 @@ bool UpdateModel::setData(const QModelIndex &idx, const QVariant &value, int rol
         Q_EMIT toUpdateChanged();
 
         return true;
+    } else if (role == ExtendedRole) {
+        UpdateItem *item = itemFromIndex(idx);
+        if (item->isExtended() != value.toBool()) {
+            item->setExtended(value.toBool());
+            Q_EMIT dataChanged(idx, idx, {ExtendedRole});
+        }
     }
 
     return false;
@@ -305,7 +314,13 @@ UpdateItem *UpdateModel::itemFromResource(AbstractResource *res)
 
 QString UpdateModel::updateSize() const
 {
-    return m_updates ? KFormat().formatByteSize(m_updates->updateSize()) : QString();
+    if (!m_updates) {
+        return QString();
+    }
+    if (m_updates->updateSize() != 0) {
+        return KFormat().formatByteSize(m_updates->updateSize());
+    }
+    return i18n("Unknown");
 }
 
 QModelIndex UpdateModel::indexFromItem(UpdateItem *item) const
@@ -335,14 +350,36 @@ void UpdateModel::resourceDataChanged(AbstractResource *res, const QVector<QByte
 
 void UpdateModel::checkAll()
 {
-    for (int i = 0, c = rowCount(); i < c; ++i)
-        if (index(i, 0).data(Qt::CheckStateRole) != Qt::Checked)
-            setData(index(i, 0), Qt::Checked, Qt::CheckStateRole);
+    QList<AbstractResource *> updatedItems;
+
+    for (int i = 0, c = rowCount(); i < c; ++i) {
+        auto idx = index(i);
+        if (idx.data(Qt::CheckStateRole) != Qt::Checked) {
+            updatedItems.append(itemFromIndex(idx)->app());
+        }
+    }
+
+    checkResources(updatedItems, true);
+
+    Q_EMIT dataChanged(index(0), index(rowCount() - 1), {Qt::CheckStateRole});
+    Q_EMIT toUpdateChanged();
 }
 
 void UpdateModel::uncheckAll()
 {
-    for (int i = 0, c = rowCount(); i < c; ++i)
-        if (index(i, 0).data(Qt::CheckStateRole) != Qt::Unchecked)
-            setData(index(i, 0), Qt::Unchecked, Qt::CheckStateRole);
+    QList<AbstractResource *> updatedItems;
+
+    for (int i = 0, c = rowCount(); i < c; ++i) {
+        auto idx = index(i);
+        if (idx.data(Qt::CheckStateRole) != Qt::Unchecked) {
+            updatedItems.append(itemFromIndex(idx)->app());
+        }
+    }
+
+    checkResources(updatedItems, false);
+
+    Q_EMIT dataChanged(index(0), index(rowCount() - 1), {Qt::CheckStateRole});
+    Q_EMIT toUpdateChanged();
 }
+
+#include "moc_UpdateModel.cpp"
