@@ -4,8 +4,7 @@
  *
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
-#ifndef FLATPAKNOTIFIER_H
-#define FLATPAKNOTIFIER_H
+#pragma once
 
 #include <BackendNotifierModule.h>
 #include <functional>
@@ -50,6 +49,5 @@ public:
     Installation m_user;
     Installation m_system;
     GCancellable *const m_cancellable;
+    bool m_lastHasUpdates = false;
 };
-
-#endif
