@@ -1,5 +1,6 @@
 /*
  *   SPDX-FileCopyrightText: 2017 Jan Grulich <jgrulich@redhat.com>
+ *   SPDX-FileCopyrightText: 2023 Harald Sitter <sitter@kde.org>
  *
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
@@ -10,6 +11,7 @@
 #include <KLocalizedString>
 #include <QDebug>
 #include <QDesktopServices>
+#include <QScopeGuard>
 
 static int FLATPAK_CLI_UPDATE_FREQUENCY = 150;
 
@@ -62,7 +64,9 @@ void FlatpakTransactionThread::new_operation_cb(FlatpakTransaction * /*object*/,
 void operation_error_cb(FlatpakTransaction * /*object*/, FlatpakTransactionOperation * /*operation*/, GError *error, gint /*details*/, gpointer user_data)
 {
     FlatpakTransactionThread *obj = (FlatpakTransactionThread *)user_data;
-    obj->addErrorMessage(QString::fromUtf8(error->message));
+    if (error) {
+        obj->addErrorMessage(QString::fromUtf8(error->message));
+    }
 }
 
 gboolean
@@ -90,8 +94,7 @@ void FlatpakTransactionThread::webflowDoneCallback(FlatpakTransaction *transacti
 }
 
 FlatpakTransactionThread::FlatpakTransactionThread(FlatpakResource *app, Transaction::Role role)
-    : QThread()
-    , m_result(false)
+    : m_result(false)
     , m_app(app)
     , m_role(role)
 {
@@ -129,6 +132,10 @@ void FlatpakTransactionThread::cancel()
 
 void FlatpakTransactionThread::run()
 {
+    auto finish = qScopeGuard([this] {
+        Q_EMIT finished();
+    });
+
     if (!m_transaction)
         return;
     g_autoptr(GError) localError = nullptr;
@@ -193,8 +200,11 @@ void FlatpakTransactionThread::run()
 #if defined(FLATPAK_LIST_UNUSED_REFS)
     } else {
         const auto installation = flatpak_transaction_get_installation(m_transaction);
-        g_autoptr(GPtrArray) refs = flatpak_installation_list_unused_refs(installation, nullptr, m_cancellable, nullptr);
-        if (refs->len > 0) {
+        g_autoptr(GError) refsError = nullptr;
+        g_autoptr(GPtrArray) refs = flatpak_installation_list_unused_refs(installation, nullptr, m_cancellable, &refsError);
+        if (!refs) {
+            qWarning() << "could not fetch unused refs" << refsError->message;
+        } else if (refs->len > 0) {
             g_autoptr(GError) localError = nullptr;
             qDebug() << "found unused refs:" << refs->len;
             auto transaction = flatpak_transaction_new_for_installation(installation, m_cancellable, &localError);
