@@ -33,6 +33,7 @@
 #include "KNSBackend.h"
 #include "KNSResource.h"
 #include "KNSReviews.h"
+#include "KNSTransaction.h"
 #include "utils.h"
 #include <resources/StandardBackendUpdater.h>
 
@@ -46,9 +47,30 @@ class KNSBackendFactory : public AbstractResourcesBackendFactory
 public:
     KNSBackendFactory()
     {
-        connect(KNSCore::QuestionManager::instance(), &KNSCore::QuestionManager::askQuestion, this, [](KNSCore::Question *q) {
-            qWarning() << q->question() << q->questionType();
-            q->setResponse(KNSCore::Question::InvalidResponse);
+        connect(KNSCore::QuestionManager::instance(), &KNSCore::QuestionManager::askQuestion, this, [](KNSCore::Question *question) {
+            const auto transactions = TransactionModel::global()->transactions();
+            for (auto t : transactions) {
+                const auto transaction = dynamic_cast<KNSTransaction *>(t);
+                if (!transaction) {
+                    continue;
+                }
+
+                if (question->entry().uniqueId() == transaction->uniqueId()) {
+                    switch (question->questionType()) {
+                    case KNSCore::Question::ContinueCancelQuestion:
+                        transaction->addQuestion(question);
+                        return;
+                    default:
+                        transaction->passiveMessage(i18n("Unsupported question:\n%1", question->question()));
+                        question->setResponse(KNSCore::Question::InvalidResponse);
+                        transaction->setStatus(Transaction::CancelledStatus);
+                        break;
+                    }
+                    return;
+                }
+            }
+            qWarning() << "Question for unknown resource" << question->question() << question->questionType();
+            question->setResponse(KNSCore::Question::InvalidResponse);
         });
     }
 
@@ -67,7 +89,9 @@ public:
     }
 };
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 Q_DECLARE_METATYPE(KNSCore::EntryInternal)
+#endif
 
 KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &knsrc)
     : AbstractResourcesBackend(parent)
@@ -93,10 +117,6 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
     const KConfigGroup group = conf.group("KNewStuff3");
     m_extends = group.readEntry("Extends", QStringList());
 
-#if KNEWSTUFFCORE_VERSION < QT_VERSION_CHECK(5, 92, 0)
-    m_reviews->setProviderUrl(QUrl(group.readEntry("ProvidersUrl", QString())));
-#endif
-
     setFetching(true);
 
     // This ensures we have something to track when checking after the initialization timeout
@@ -111,7 +131,7 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
         }
     });
 
-    const QVector<QPair<FilterType, QString>> filters = {{CategoryFilter, fileName}};
+    const CategoryFilter filter = {CategoryFilter::CategoryNameFilter, fileName};
     const QSet<QString> backendName = {name()};
     m_displayName = group.readEntry("Name", QString());
     if (m_displayName.isEmpty()) {
@@ -126,9 +146,9 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
         m_categories += cats;
         for (const auto &cat : cats) {
             if (m_hasApplications)
-                categories << new Category(cat, QStringLiteral("applications-other"), {{CategoryFilter, cat}}, backendName, {}, {}, true);
+                categories << new Category(cat, QStringLiteral("applications-other"), {CategoryFilter::CategoryNameFilter, cat}, backendName, {}, true);
             else
-                categories << new Category(cat, QStringLiteral("plasma"), {{CategoryFilter, cat}}, backendName, {}, {}, true);
+                categories << new Category(cat, QStringLiteral("plasma"), {CategoryFilter::CategoryNameFilter, cat}, backendName, {}, true);
         }
     }
 
@@ -156,7 +176,7 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
     connect(m_engine, &KNSCore::Engine::signalCategoriesMetadataLoded, this, [categories](const QList<KNSCore::Provider::CategoryMetadata> &categoryMetadatas) {
         for (const KNSCore::Provider::CategoryMetadata &category : categoryMetadatas) {
             for (Category *cat : qAsConst(categories)) {
-                if (cat->orFilters().count() > 0 && cat->orFilters().constFirst().second == category.name) {
+                if (cat->matchesCategoryName(category.name)) {
                     cat->setName(category.displayName);
                     break;
                 }
@@ -167,15 +187,15 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
     m_engine->init(m_name);
 
     if (m_hasApplications) {
-        auto actualCategory = new Category(m_displayName, QStringLiteral("applications-other"), filters, backendName, topCategories, QUrl(), false);
+        auto actualCategory = new Category(m_displayName, QStringLiteral("applications-other"), filter, backendName, topCategories, false);
         auto applicationCategory = new Category(i18n("Applications"), //
                                                 QStringLiteral("applications-internet"),
-                                                filters,
+                                                filter,
                                                 backendName,
                                                 {actualCategory},
-                                                QUrl(),
                                                 false);
-        applicationCategory->setAndFilter({{CategoryFilter, QLatin1String("Application")}});
+        const QVector<CategoryFilter> filters = {{CategoryFilter::CategoryNameFilter, QLatin1String("Application")}, filter};
+        applicationCategory->setFilter({CategoryFilter::AndFilter, filters});
         m_categories.append(applicationCategory->name());
         m_rootCategories = {applicationCategory};
         // Make sure we filter out any apps which won't run on the current system architecture
@@ -211,10 +231,11 @@ KNSBackend::KNSBackend(QObject *parent, const QString &iconName, const QString &
             QStringLiteral("ksplash.knsrc"),       QStringLiteral("window-decorations.knsrc"),
         };
         const auto iconName = knsrcPlasma.contains(fileName) ? QStringLiteral("plasma") : QStringLiteral("applications-other");
-        auto actualCategory = new Category(m_displayName, iconName, filters, backendName, categories, QUrl(), true);
+        auto actualCategory = new Category(m_displayName, iconName, filter, backendName, categories, true);
+        actualCategory->setParent(this);
 
         const auto topLevelName = knsrcPlasma.contains(fileName) ? i18n("Plasma Addons") : i18n("Application Addons");
-        auto addonsCategory = new Category(topLevelName, iconName, filters, backendName, {actualCategory}, QUrl(), true);
+        auto addonsCategory = new Category(topLevelName, iconName, filter, backendName, {actualCategory}, true);
         m_rootCategories = {addonsCategory};
     }
 
@@ -268,7 +289,7 @@ void KNSBackend::fetchInstalled()
             auto updateChecker = new OneTimeAction(
                 666,
                 [this] {
-                    //No need to check for updates if there's no resources
+                    // No need to check for updates if there's no resources
                     if (m_resourcesByName.isEmpty()) {
                         return true;
                     }
@@ -495,85 +516,6 @@ void KNSBackend::slotEntryEvent(const KNSCore::EntryInternal &entry, KNSCore::En
     }
 }
 
-class KNSTransaction : public Transaction
-{
-public:
-    KNSTransaction(QObject *parent, KNSResource *res, Transaction::Role role)
-        : Transaction(parent, res, role)
-        , m_id(res->entry().uniqueId())
-    {
-        setCancellable(false);
-
-        auto manager = res->knsBackend()->engine();
-        connect(manager, &KNSCore::Engine::signalEntryEvent, this, [this](const KNSCore::EntryInternal &entry, KNSCore::EntryInternal::EntryEvent event) {
-            switch (event) {
-            case KNSCore::EntryInternal::StatusChangedEvent:
-                anEntryChanged(entry);
-                break;
-            case KNSCore::EntryInternal::DetailsLoadedEvent:
-            case KNSCore::EntryInternal::AdoptedEvent:
-            case KNSCore::EntryInternal::UnknownEvent:
-            default:
-                break;
-            }
-        });
-        TransactionModel::global()->addTransaction(this);
-
-        std::function<void()> actionFunction;
-        auto engine = res->knsBackend()->engine();
-        if (role == RemoveRole)
-            actionFunction = [res, engine]() {
-                engine->uninstall(res->entry());
-            };
-        else if (res->entry().status() == KNS3::Entry::Updateable)
-            actionFunction = [res, engine]() {
-                engine->install(res->entry(), -1);
-            };
-        else if (res->linkIds().isEmpty())
-            actionFunction = [res]() {
-                qWarning() << "No installable candidates in the KNewStuff entry" << res->entry().name() << "with id" << res->entry().uniqueId()
-                           << "on the backend" << res->backend()->name()
-                           << "There should always be at least one downloadable item in an OCS entry, and if there isn't, we should consider it broken. OCS "
-                              "can technically show them, but if there is nothing to install, it cannot be installed.";
-            };
-        else
-            actionFunction = [res, engine]() {
-                engine->install(res->entry());
-            };
-        QTimer::singleShot(0, res, actionFunction);
-    }
-
-    void anEntryChanged(const KNSCore::EntryInternal &entry)
-    {
-        if (entry.uniqueId() == m_id) {
-            switch (entry.status()) {
-            case KNS3::Entry::Invalid:
-                qWarning() << "invalid status for" << entry.uniqueId() << entry.status();
-                break;
-            case KNS3::Entry::Installing:
-            case KNS3::Entry::Updating:
-                setStatus(CommittingStatus);
-                break;
-            case KNS3::Entry::Downloadable:
-            case KNS3::Entry::Installed:
-            case KNS3::Entry::Deleted:
-            case KNS3::Entry::Updateable:
-                if (status() != DoneStatus) {
-                    setStatus(DoneStatus);
-                }
-                break;
-            }
-        }
-    }
-
-    void cancel() override
-    {
-    }
-
-private:
-    const QString m_id;
-};
-
 Transaction *KNSBackend::removeApplication(AbstractResource *app)
 {
     auto res = qobject_cast<KNSResource *>(app);
@@ -679,7 +621,7 @@ void KNSBackend::searchStream(ResultsStream *stream, const QString &searchText)
     if (m_responsePending) {
         connect(this, &KNSBackend::availableForQueries, stream, start, Qt::QueuedConnection);
     } else if (isFetching()) {
-        connect(this, &KNSBackend::initialized, stream, start);
+        connect(this, &KNSBackend::initialized, stream, start, Qt::QueuedConnection);
         connect(this, &KNSBackend::availableForQueries, stream, start, Qt::QueuedConnection);
     } else {
         QTimer::singleShot(0, stream, start);
