@@ -18,21 +18,6 @@
 #include <attica/providermanager.h>
 #include <resources/AbstractResource.h>
 
-class SharedManager : public QObject
-{
-    Q_OBJECT
-public:
-    SharedManager()
-    {
-        atticaManager.loadDefaultProviders();
-    }
-
-public:
-    Attica::ProviderManager atticaManager;
-};
-
-Q_GLOBAL_STATIC(SharedManager, s_shared)
-
 KNSReviews::KNSReviews(KNSBackend *backend)
     : AbstractReviewsBackend(backend)
     , m_backend(backend)
@@ -60,7 +45,20 @@ void KNSReviews::fetchReviews(AbstractResource *app, int page)
     job->setProperty("app", QVariant::fromValue<AbstractResource *>(app));
     connect(job, &Attica::BaseJob::finished, this, &KNSReviews::commentsReceived);
     job->start();
-    m_fetching++;
+    acquireFetching(true);
+}
+
+void KNSReviews::acquireFetching(bool f)
+{
+    if (f)
+        m_fetching++;
+    else
+        m_fetching--;
+
+    if ((!f && m_fetching == 0) || (f && m_fetching == 1)) {
+        Q_EMIT fetchingChanged(m_fetching != 0);
+    }
+    Q_ASSERT(m_fetching >= 0);
 }
 
 static QVector<ReviewPtr> createReviewList(AbstractResource *app, const Attica::Comment::List comments, int depth = 0)
@@ -92,7 +90,7 @@ static QVector<ReviewPtr> createReviewList(AbstractResource *app, const Attica::
 
 void KNSReviews::commentsReceived(Attica::BaseJob *j)
 {
-    m_fetching--;
+    acquireFetching(false);
     Attica::ListJob<Attica::Comment> *job = static_cast<Attica::ListJob<Attica::Comment> *>(j);
 
     AbstractResource *app = job->property("app").value<AbstractResource *>();
@@ -116,8 +114,9 @@ void KNSReviews::deleteReview(Review * /*r*/)
     qWarning() << "cannot delete comments";
 }
 
-void KNSReviews::submitReview(AbstractResource *app, const QString &summary, const QString &review_text, const QString &rating)
+void KNSReviews::sendReview(AbstractResource *app, const QString &summary, const QString &review_text, const QString &rating, const QString &userName)
 {
+    Q_UNUSED(userName);
     provider().voteForContent(app->packageName(), rating.toUInt() * 20);
     if (!summary.isEmpty())
         provider().addNewComment(Attica::Comment::ContentComment, app->packageName(), QString(), QString(), summary, review_text);
@@ -166,28 +165,15 @@ QString KNSReviews::userName() const
     return user;
 }
 
-void KNSReviews::setProviderUrl(const QUrl &url)
-{
-#if KNEWSTUFFCORE_VERSION < QT_VERSION_CHECK(5, 92, 0)
-    m_providerUrl = url;
-    if (!m_providerUrl.isEmpty() && !s_shared->atticaManager.providerFiles().contains(url)) {
-        s_shared->atticaManager.addProviderFile(url);
-    }
-#endif
-}
-
 Attica::Provider KNSReviews::provider() const
 {
-#if KNEWSTUFFCORE_VERSION < QT_VERSION_CHECK(5, 92, 0)
-    return s_shared->atticaManager.providerFor(m_providerUrl);
-#else
+    if (m_backend->engine()->atticaProviders().isEmpty()) {
+        return {};
+    }
     return *m_backend->engine()->atticaProviders().constFirst();
-#endif
 }
 
 bool KNSReviews::isResourceSupported(AbstractResource *res) const
 {
     return qobject_cast<KNSResource *>(res);
 }
-
-#include "KNSReviews.moc"
