@@ -4,10 +4,10 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef ABSTRACTRESOURCE_H
-#define ABSTRACTRESOURCE_H
+#pragma once
 
 #include <QCollatorSortKey>
+#include <QColor>
 #include <QDate>
 #include <QJsonArray>
 #include <QJsonObject>
@@ -25,6 +25,27 @@ class Category;
 class Rating;
 class AbstractResourcesBackend;
 
+struct Screenshot {
+    Screenshot(const QUrl &screenshot)
+        : thumbnail(screenshot)
+        , screenshot(screenshot)
+    {
+    }
+
+    Screenshot(const QUrl &thumbnail, const QUrl &screenshot, bool isAnimated)
+        : thumbnail(thumbnail)
+        , screenshot(screenshot)
+        , isAnimated(isAnimated)
+    {
+    }
+
+    QUrl thumbnail;
+    QUrl screenshot;
+    bool isAnimated = false;
+};
+
+using Screenshots = QVector<Screenshot>;
+
 /**
  * \class AbstractResource  AbstractResource.h "AbstractResource.h"
  *
@@ -47,6 +68,7 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(QUrl helpURL READ helpURL CONSTANT)
     Q_PROPERTY(QUrl bugURL READ bugURL CONSTANT)
     Q_PROPERTY(QUrl donationURL READ donationURL CONSTANT)
+    Q_PROPERTY(QUrl contributeURL READ contributeURL CONSTANT)
     Q_PROPERTY(bool canUpgrade READ canUpgrade NOTIFY stateChanged)
     Q_PROPERTY(bool isInstalled READ isInstalled NOTIFY stateChanged)
     Q_PROPERTY(QJsonArray licenses READ licenses NOTIFY licensesChanged)
@@ -59,10 +81,9 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(QString availableVersion READ availableVersion NOTIFY versionsChanged)
     Q_PROPERTY(QString section READ section CONSTANT)
     Q_PROPERTY(QStringList mimetypes READ mimetypes CONSTANT)
-    Q_PROPERTY(AbstractResourcesBackend *backend READ backend CONSTANT)
+    Q_PROPERTY(QObject *backend READ backendObject CONSTANT)
     Q_PROPERTY(QVariant rating READ ratingVariant NOTIFY ratingFetched)
     Q_PROPERTY(QString appstreamId READ appstreamId CONSTANT)
-    Q_PROPERTY(QString categoryDisplay READ categoryDisplay CONSTANT)
     Q_PROPERTY(QUrl url READ url CONSTANT)
     Q_PROPERTY(QString executeLabel READ executeLabel CONSTANT)
     Q_PROPERTY(QString sourceIcon READ sourceIcon CONSTANT)
@@ -71,6 +92,10 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(QString upgradeText READ upgradeText NOTIFY versionsChanged)
     Q_PROPERTY(bool isRemovable READ isRemovable CONSTANT)
     Q_PROPERTY(QString versionString READ versionString NOTIFY versionsChanged)
+    Q_PROPERTY(QString contentRatingText READ contentRatingText CONSTANT)
+    Q_PROPERTY(QString contentRatingDescription READ contentRatingDescription CONSTANT)
+    Q_PROPERTY(ContentIntensity contentRatingIntensity READ contentRatingIntensity CONSTANT)
+    Q_PROPERTY(uint contentRatingMinimumAge READ contentRatingMinimumAge CONSTANT)
 public:
     /**
      * This describes the state of the resource
@@ -95,6 +120,12 @@ public:
     };
     Q_ENUM(State)
 
+    enum ContentIntensity {
+        Mild,
+        Intense,
+    };
+    Q_ENUM(ContentIntensity)
+
     /**
      * Constructs the AbstractResource with its corresponding backend
      */
@@ -131,13 +162,15 @@ public:
     virtual QUrl bugURL();
     ///@returns a URL that points to the place where you can donate money to the app developer
     virtual QUrl donationURL();
+    ///@returns a URL that points to the place where you can contribute to develop the app
+    virtual QUrl contributeURL();
 
     enum Type {
         Application,
         Addon,
         Technical,
     };
-    Q_ENUM(Type);
+    Q_ENUM(Type)
     virtual Type type() const = 0;
 
     virtual quint64 size() = 0;
@@ -175,6 +208,7 @@ public:
     virtual QString status();
 
     AbstractResourcesBackend *backend() const;
+    QObject *backendObject() const;
 
     /**
      * @returns a name sort key for faster sorting
@@ -189,11 +223,6 @@ public:
     Rating *rating() const;
     QVariant ratingVariant() const;
 
-    /**
-     * @returns a string defining the categories the resource belongs to
-     */
-    QString categoryDisplay() const;
-
     bool categoryMatches(Category *cat);
 
     QSet<Category *> categoryObjects(const QVector<Category *> &cats) const;
@@ -227,6 +256,11 @@ public:
 
     virtual QString versionString();
 
+    virtual QString contentRatingText() const;
+    virtual ContentIntensity contentRatingIntensity() const;
+    virtual QString contentRatingDescription() const;
+    virtual uint contentRatingMinimumAge() const;
+
 public Q_SLOTS:
     virtual void fetchScreenshots();
     virtual void fetchChangelog() = 0;
@@ -245,8 +279,7 @@ Q_SIGNALS:
     void versionsChanged();
 
     /// response to the fetchScreenshots method
-    ///@p thumbnails and @p screenshots should have the same number of elements
-    void screenshotsFetched(const QList<QUrl> &thumbnails, const QList<QUrl> &screenshots);
+    void screenshotsFetched(const Screenshots &screenshots);
     void changelogFetched(const QString &changelog);
 
 private:
@@ -258,5 +291,3 @@ private:
 };
 
 Q_DECLARE_METATYPE(QVector<AbstractResource *>)
-
-#endif // ABSTRACTRESOURCE_H
